@@ -116,8 +116,8 @@ class Pyparticles:
                         beta0  = {beta0}"""
                         )
 
-    def __init__delta(self, delta, ptau, psigma, mask_check=None):
-        not_none = count_not_none(delta, ptau, psigma)
+    def __init__delta(self, delta, ptau, pzeta, mask_check=None):
+        not_none = count_not_none(delta, ptau, pzeta)
         if not_none == 0:
             self.delta = 0.0
         elif not_none >= 1:
@@ -125,11 +125,11 @@ class Pyparticles:
                 self.delta = delta
             elif ptau is not None:
                 self.ptau = ptau
-            elif psigma is not None:
-                self.psigma = psigma
+            elif pzeta is not None:
+                self.pzeta = pzeta
 
             if not_none>1:
-                ddd = {'delta': delta, 'ptau': ptau, 'psigma': psigma}
+                ddd = {'delta': delta, 'ptau': ptau, 'pzeta': pzeta}
                 for nn, vv in ddd.items():
                     if vv is None:
                         continue
@@ -146,11 +146,11 @@ class Pyparticles:
                         Particles defined with inconsistent energy deviations:
                         delta  = {delta},
                         ptau     = {ptau},
-                        psigma = {psigma}"""
+                        pzeta = {pzeta}"""
                         )
 
-    def __init__zeta(self, zeta, tau, sigma):
-        not_none = count_not_none(zeta, tau, sigma)
+    def __init__zeta(self, zeta, tau):
+        not_none = count_not_none(zeta, tau)
         if not_none == 0:
             self.zeta = 0.0
         elif not_none == 1:
@@ -158,15 +158,12 @@ class Pyparticles:
                 self.zeta = zeta
             elif tau is not None:
                 self.tau = tau
-            elif sigma is not None:
-                self.sigma = sigma
         else:
             raise ValueError(
                 f"""\
             Particles defined with multiple time deviations:
             zeta  = {zeta},
-            tau   = {tau},
-            sigma = {sigma}"""
+            tau   = {tau}"""
             )
 
     def __init__chi(self, mass_ratio, charge_ratio, chi):
@@ -227,11 +224,10 @@ class Pyparticles:
         py = kwargs.get('py', 0.0)
         delta = kwargs.get('delta', None)
         ptau = kwargs.get('ptau', None)
-        psigma = kwargs.get('psigma', None)
+        pzeta = kwargs.get('pzeta', None)
         rvv = kwargs.get('rvv', None)
         zeta = kwargs.get('zeta', None)
         tau = kwargs.get('tau', None)
-        sigma = kwargs.get('sigma', None)
         mass0 = kwargs.get('mass0', pmass)
         q0 = kwargs.get('q0', 1.0)
         p0c = kwargs.get('p0c', None)
@@ -267,8 +263,8 @@ class Pyparticles:
         else:
             mask_check = None
         self.__init__ref(p0c, energy0, gamma0, beta0, mask_check=mask_check)
-        self.__init__delta(delta, ptau, psigma, mask_check=mask_check)
-        self.__init__zeta(zeta, tau, sigma)
+        self.__init__delta(delta, ptau, pzeta, mask_check=mask_check)
+        self.__init__zeta(zeta, tau)
         self.__init__chi(chi=chi, mass_ratio=mass_ratio, charge_ratio=charge_ratio)
         self._update_coordinates = True
         length = self._check_array_length()
@@ -329,7 +325,6 @@ class Pyparticles:
 
     def add_to_energy(self, energy):
         sqrt = self._m.sqrt
-        oldrvv = self._rvv
         deltabeta0 = self.delta * self.beta0
         ptaubeta0 = sqrt(deltabeta0 ** 2 + 2 * deltabeta0 * self.beta0 + 1) - 1
         ptaubeta0 += energy / self.energy0
@@ -337,43 +332,29 @@ class Pyparticles:
         self._delta = sqrt(ptau ** 2 + 2 * ptau / self.beta0 + 1) - 1
         self._rvv = (1 + self.delta) / (1 + ptaubeta0)
         self._rpp = 1 / (1 + self.delta)
-        self.zeta *= self._rvv / oldrvv
 
     delta = property(lambda self: self._delta)
 
     @delta.setter
     def delta(self, delta):
         sqrt = self._m.sqrt
-        if hasattr(self, '_rvv'):
-            oldrvv = self._rvv
-        else:
-            # Needed for particle initialization
-            oldrvv = None
         self._delta = delta
         deltabeta0 = delta * self.beta0
         ptaubeta0 = sqrt(deltabeta0 ** 2 + 2 * deltabeta0 * self.beta0 + 1) - 1
         self._rvv = (1 + self.delta) / (1 + ptaubeta0)
         self._rpp = 1 / (1 + self.delta)
-        if oldrvv is not None:
-            self.zeta *= self._rvv / oldrvv
 
-    psigma = property(lambda self: self.ptau / self.beta0)
+    pzeta = property(lambda self: self.ptau / self.beta0)
 
-    @psigma.setter
-    def psigma(self, psigma):
-        self.ptau = psigma * self.beta0
+    @pzeta.setter
+    def pzeta(self, pzeta):
+        self.ptau = pzeta * self.beta0
 
-    tau = property(lambda self: self.zeta / self.beta)
+    tau = property(lambda self: self.zeta / self.beta0)
 
     @tau.setter
     def tau(self, tau):
-        self.zeta = self.beta * tau
-
-    sigma = property(lambda self: (self.beta0 / self.beta) * self.zeta)
-
-    @sigma.setter
-    def sigma(self, sigma):
-        self.zeta = self.beta / self.beta0 * sigma
+        self.zeta = self.beta0 * tau
 
     @property
     def ptau(self):
@@ -455,7 +436,7 @@ class Pyparticles:
         self._chi = chi
 
     def _get_absolute(self):
-        return self.Px, self.Py, self.pc, self.energy
+        return self.Px, self.Py, self.pc, self.energy, self.tau
 
     def _update_ref(self, mass0, beta0, gamma0, p0c, energy0):
         self._mass0 = mass0
@@ -464,7 +445,7 @@ class Pyparticles:
         self._p0c = p0c
         self._energy0 = energy0
 
-    def _update_particles_from_absolute(self, Px, Py, pc, energy):
+    def _update_particles_from_absolute(self, Px, Py, pc, energy, tau):
         if self._update_coordinates:
             mass_ratio = self.mass / self.mass0
             norm = mass_ratio * self.p0c
@@ -474,6 +455,7 @@ class Pyparticles:
             self._delta = pc / norm - 1
             self.px = Px / norm
             self.py = Py / norm
+            self.zeta = tau * self.beta0
 
     def __repr__(self):
         out = f"""\
@@ -618,4 +600,4 @@ class Pyparticles:
         for ii, pp in enumerate(lst):
             for nn in cls._dict_vars:
                 dct[nn][ii] = getattr(pp, nn, 0)
-        return cls(**dct)
+        return cls(**dct)
\ No newline at end of file
